@@ -279,7 +279,6 @@ drag_data_get_cb (GtkWidget          *widget,
 		  guint               time,
 		  Launcher           *launcher)
 {
-	char *uri_list;
 	const char *location;
 	
 	g_return_if_fail (launcher != NULL);
@@ -287,25 +286,33 @@ drag_data_get_cb (GtkWidget          *widget,
 
 	location = gnome_desktop_item_get_location (launcher->ditem);
 
-	if (location == NULL)
+	if (!location) {
 		launcher_save (launcher);
+		location = gnome_desktop_item_get_location (launcher->ditem);
+	}
 
 	if (info == TARGET_URI_LIST) {
-		uri_list = g_strconcat ("file:", location, "\r\n", NULL);
+		if (location [0] != '/')
+			gtk_selection_data_set (
+				selection_data, selection_data->target, 8,
+				(guchar *)location, strlen (location));
+		else {
+			char *uri;
 
-		gtk_selection_data_set (selection_data,
-					selection_data->target, 8, (guchar *)uri_list,
-					strlen (uri_list));
-		g_free(uri_list);
-	} else if (info == TARGET_ICON_INTERNAL) {
+			uri = g_strconcat ("file://", location, "\r\n", NULL);
+
+			gtk_selection_data_set (
+				selection_data, selection_data->target, 8,
+				(guchar *)uri, strlen (uri));
+
+			g_free (uri);
+		}
+	} else if (info == TARGET_ICON_INTERNAL)
 		gtk_selection_data_set (selection_data,
 					selection_data->target, 8,
 					location, strlen (location));
-	}
 }
 
-
-
 static Launcher *
 create_launcher (const char *parameters, GnomeDesktopItem *ditem)
 {
@@ -327,7 +334,6 @@ create_launcher (const char *parameters, GnomeDesktopItem *ditem)
 							 0 /* flags */,
 							 NULL /* error */);
 
-		
 		if (ditem == NULL) {
 			gchar *entry;
 
@@ -656,7 +662,7 @@ launcher_save_to_gconf (Launcher   *launcher,
 		profile = panel_gconf_get_profile ();
 
 		temp_key = panel_gconf_full_key (PANEL_GCONF_OBJECTS, profile,
-						 gconf_key, "launcher-location");
+						 gconf_key, "launcher_location");
 		gconf_client_set_string (client, temp_key, location, NULL);
 	}
 }
@@ -671,14 +677,14 @@ launcher_load_from_gconf (PanelWidget *panel_widget,
 	const char  *temp_key;
 	char        *launcher_location;
 
-	g_return_if_fail (panel_widget);
-	g_return_if_fail (gconf_key);
+	g_return_if_fail (panel_widget != NULL);
+	g_return_if_fail (gconf_key != NULL);
 
 	client  = panel_gconf_get_client ();
 	profile = panel_gconf_get_profile ();
 
 	temp_key = panel_gconf_full_key (
-			PANEL_GCONF_OBJECTS, profile, gconf_key, "launcher-location");
+			PANEL_GCONF_OBJECTS, profile, gconf_key, "launcher_location");
 	launcher_location = gconf_client_get_string (client, temp_key, NULL);
 
 	load_launcher_applet (launcher_location, panel_widget, position, TRUE, gconf_key);
@@ -973,7 +979,7 @@ launcher_save (Launcher *launcher)
 	g_return_if_fail (launcher != NULL);
 	g_return_if_fail (launcher->ditem != NULL);
 
-	if (gnome_desktop_item_get_location (launcher->ditem) == NULL)
+	if (!gnome_desktop_item_get_location (launcher->ditem))
 		gnome_desktop_item_set_location (launcher->ditem,
 						 launcher_get_unique_uri ());
 
