@@ -66,6 +66,7 @@ static void panel_widget_cremove        (GtkContainer     *container,
 static int  panel_widget_expose         (GtkWidget        *widget,
 					 GdkEventExpose   *event);
 static void panel_widget_destroy        (GtkObject        *obj);
+static void panel_widget_finalize       (GObject        *obj);
 static void panel_widget_style_set      (GtkWidget        *widget,
 					 GtkStyle         *previous_style);
 static void panel_widget_realize        (GtkWidget        *widget);
@@ -84,6 +85,8 @@ static void panel_widget_free_move_applet   (PanelWidget      *panel,
 static void panel_widget_tab_move           (PanelWidget      *panel,
                                              gboolean          next);
 static void panel_widget_end_move           (PanelWidget      *panel);
+static gboolean panel_widget_real_focus     (GtkWidget        *widget,
+                                             GtkDirectionType  direction);
 
 
 typedef void (*BackSignal) (GtkObject * object,
@@ -227,10 +230,9 @@ add_all_move_bindings (PanelWidget *panel)
 	add_move_bindings (binding_set, GDK_CONTROL_MASK, "switch_move");
 	add_move_bindings (binding_set, GDK_MOD1_MASK, "free_move");
 
-	if (!GTK_WIDGET_HAS_FOCUS (panel)) {
-		add_tab_bindings (binding_set, 0, TRUE);
-		add_tab_bindings (binding_set, GDK_SHIFT_MASK, FALSE);
-	}
+	add_tab_bindings (binding_set, 0, TRUE);
+	add_tab_bindings (binding_set, GDK_SHIFT_MASK, FALSE);
+
 	gtk_binding_entry_add_signal (binding_set,
                                       GDK_Escape, 0,
                                       "end_move", 0);
@@ -269,10 +271,9 @@ remove_all_move_bindings (PanelWidget *panel)
 	remove_move_bindings (binding_set, GDK_SHIFT_MASK);
 	remove_move_bindings (binding_set, GDK_CONTROL_MASK);
 	remove_move_bindings (binding_set, GDK_MOD1_MASK);
-	if (!GTK_WIDGET_HAS_FOCUS (panel)) {
-		remove_tab_bindings (binding_set, 0, TRUE);
-		remove_tab_bindings (binding_set, GDK_SHIFT_MASK, FALSE);
-	}
+	remove_tab_bindings (binding_set, 0, TRUE);
+	remove_tab_bindings (binding_set, GDK_SHIFT_MASK, FALSE);
+
 	gtk_binding_entry_clear (binding_set, GDK_Escape, 0);
 }
 
@@ -280,11 +281,10 @@ static void
 panel_widget_class_init (PanelWidgetClass *class)
 {
 	GtkObjectClass *object_class = (GtkObjectClass*) class;
+	GObjectClass *gobject_class = (GObjectClass*) class;
 	GtkWidgetClass *widget_class = (GtkWidgetClass*) class;
 	GtkContainerClass *container_class = (GtkContainerClass*) class;
 
-	GtkBindingSet *binding_set;
-
 	panel_widget_signals[ORIENT_CHANGE_SIGNAL] =
                 g_signal_new ("orient_change",
                               G_TYPE_FROM_CLASS (object_class),
@@ -292,7 +292,7 @@ panel_widget_class_init (PanelWidgetClass *class)
                               G_STRUCT_OFFSET (PanelWidgetClass, orient_change),
                               NULL,
                               NULL, 
-                              panel_marshal_VOID__ENUM,
+                              g_cclosure_marshal_VOID__ENUM,
                               G_TYPE_NONE,
                               1,
                               GTK_TYPE_ORIENTATION); 
@@ -352,7 +352,7 @@ panel_widget_class_init (PanelWidgetClass *class)
                               G_STRUCT_OFFSET (PanelWidgetClass, back_change),
                               NULL,
                               NULL, 
-                              panel_marshal_VOID__ENUM_POINTER_POINTER,
+                              g_cclosure_marshal_VOID__ENUM,
                               G_TYPE_NONE,
                               3,
 			      PANEL_TYPE_PANEL_BACK_TYPE,
@@ -378,7 +378,7 @@ panel_widget_class_init (PanelWidgetClass *class)
                               G_STRUCT_OFFSET (PanelWidgetClass, push_move),
                               NULL,
                               NULL,
-                              panel_marshal_VOID__ENUM,
+                              g_cclosure_marshal_VOID__ENUM,
                               G_TYPE_NONE,
                               1,
                               GTK_TYPE_DIRECTION_TYPE);
@@ -390,7 +390,7 @@ panel_widget_class_init (PanelWidgetClass *class)
                               G_STRUCT_OFFSET (PanelWidgetClass, switch_move),
                               NULL,
                               NULL,
-                              panel_marshal_VOID__ENUM,
+                              g_cclosure_marshal_VOID__ENUM,
                               G_TYPE_NONE,
                               1,
                               GTK_TYPE_DIRECTION_TYPE);
@@ -402,7 +402,7 @@ panel_widget_class_init (PanelWidgetClass *class)
                               G_STRUCT_OFFSET (PanelWidgetClass, free_move),
                               NULL,
                               NULL,
-                              panel_marshal_VOID__ENUM,
+                              g_cclosure_marshal_VOID__ENUM,
                               G_TYPE_NONE,
                               1,
                               GTK_TYPE_DIRECTION_TYPE);
@@ -444,6 +444,7 @@ panel_widget_class_init (PanelWidgetClass *class)
 	class->end_move = panel_widget_end_move;
 
 	object_class->destroy = panel_widget_destroy;
+	gobject_class->finalize = panel_widget_finalize;
 	
 	widget_class->size_request = panel_widget_size_request;
 	widget_class->size_allocate = panel_widget_size_allocate;
@@ -452,13 +453,10 @@ panel_widget_class_init (PanelWidgetClass *class)
 	widget_class->style_set = panel_widget_style_set;
 	widget_class->focus_in_event = panel_widget_focus_in_event;
 	widget_class->focus_out_event = panel_widget_focus_out_event;
+	widget_class->focus = panel_widget_real_focus;
 
 	container_class->add = panel_widget_cadd;
 	container_class->remove = panel_widget_cremove;
-
-	binding_set = gtk_binding_set_by_class (class);
-	add_tab_bindings (binding_set, 0, TRUE);
-	add_tab_bindings (binding_set, GDK_SHIFT_MASK, FALSE);
 }
 
 static void
@@ -533,14 +531,17 @@ run_up_forbidden(PanelWidget *panel,
 
 
 static void
-panel_widget_cadd(GtkContainer *container, GtkWidget *widget)
+panel_widget_cadd (GtkContainer *container,
+		   GtkWidget    *widget)
 {
 	PanelWidget *p;
 
 	g_return_if_fail (PANEL_IS_WIDGET (container));
 	g_return_if_fail (GTK_IS_WIDGET (widget));
 
-	panel_widget_add(PANEL_WIDGET(container),widget,0);
+	panel_widget_add (PANEL_WIDGET (container), widget, 0,
+			  FALSE, FALSE, FALSE);
+
 	p = g_object_get_data (G_OBJECT(widget),
 			       PANEL_APPLET_ASSOC_PANEL_KEY);
 	if(p)
@@ -724,62 +725,69 @@ panel_widget_queue_applet_for_resize(AppletData *ad)
 
 
 static void
-panel_widget_switch_applet_right(PanelWidget *panel, GList *list)
+panel_widget_switch_applet_right (PanelWidget *panel,
+				  GList       *list)
 {
 	AppletData *ad;
 	AppletData *nad = NULL;
 
-	g_return_if_fail(PANEL_IS_WIDGET(panel));
-	g_return_if_fail(list!=NULL);
+	g_return_if_fail (PANEL_IS_WIDGET (panel));
+	g_return_if_fail (list != NULL);
 	
 	ad = list->data;
 
-	if(list->next)
+	if (ad->pos + ad->min_cells >= panel->size)
+		return;
+
+	if (list->next)
 		nad = list->next->data;
-	if(!nad || nad->pos > ad->pos+ad->min_cells) {
-		ad->pos++;
-		panel_widget_queue_applet_for_resize(ad);
+
+	if (!nad || nad->pos > ad->pos + ad->min_cells) {
+		ad->pos += MOVE_INCREMENT;
+		panel_widget_queue_applet_for_resize (ad);
 		return;
 	}
 
 	nad->pos = ad->pos;
-	ad->pos = nad->pos+nad->min_cells;
-	panel->applet_list = panel_g_list_swap_next(panel->applet_list,list);
+	ad->pos = nad->pos + nad->min_cells;
+	panel->applet_list = panel_g_list_swap_next (panel->applet_list,list);
 
-	panel_widget_queue_applet_for_resize(ad);
-	panel_widget_queue_applet_for_resize(nad);
+	panel_widget_queue_applet_for_resize (ad);
+	panel_widget_queue_applet_for_resize (nad);
 }
 
 static void
-panel_widget_switch_applet_left(PanelWidget *panel, GList *list)
+panel_widget_switch_applet_left (PanelWidget *panel,
+				 GList       *list)
 {
 	AppletData *ad;
 	AppletData *pad = NULL;
 
-	g_return_if_fail(PANEL_IS_WIDGET(panel));
-	g_return_if_fail(list!=NULL);
+	g_return_if_fail (PANEL_IS_WIDGET (panel));
+	g_return_if_fail (list != NULL);
 	
 	ad = list->data;
 
+	if (ad->pos <= 0)
+		return;
 
-	if(list->prev)
+	if (list->prev)
 		pad = list->prev->data;
 
-
-	if(!pad || pad->pos+pad->min_cells < ad->pos) {
-		ad->pos--;
-		panel_widget_queue_applet_for_resize(ad);
-		if (pad && pad->pos+pad->cells >= ad->pos)
-			panel_widget_queue_applet_for_resize(pad);
+	if (!pad || pad->pos + pad->min_cells < ad->pos) {
+		ad->pos -= MOVE_INCREMENT;
+		panel_widget_queue_applet_for_resize (ad);
+		if (pad && (pad->pos + pad->cells) >= ad->pos)
+			panel_widget_queue_applet_for_resize (pad);
 		return;
 	}
 
 	ad->pos = pad->pos;
-	pad->pos = ad->pos+ad->min_cells;
-	panel->applet_list = panel_g_list_swap_prev(panel->applet_list,list);
+	pad->pos = ad->pos + ad->min_cells;
+	panel->applet_list = panel_g_list_swap_prev (panel->applet_list, list);
 
-	panel_widget_queue_applet_for_resize(ad);
-	panel_widget_queue_applet_for_resize(pad);
+	panel_widget_queue_applet_for_resize (ad);
+	panel_widget_queue_applet_for_resize (pad);
 }
 
 static int
@@ -1435,7 +1443,7 @@ panel_widget_set_back_color(PanelWidget *panel, GdkColor *color)
 	panel_try_to_set_back_color(panel, color);
 	gtk_widget_queue_draw (GTK_WIDGET (panel));
 
-	g_signal_emit(GTK_OBJECT(panel),
+	g_signal_emit(G_OBJECT(panel),
 		      panel_widget_signals[BACK_CHANGE_SIGNAL],
 		      0,
 		      panel->back_type,
@@ -1605,6 +1613,30 @@ panel_widget_realize(GtkWidget *w)
 #endif
 }
 
+static void
+panel_widget_finalize (GObject *obj)
+{
+	PanelWidget *panel;
+	GtkWidget *w = GTK_WIDGET (obj);
+
+	g_return_if_fail(PANEL_IS_WIDGET(w));
+
+	panel = PANEL_WIDGET(w);
+
+	g_free (panel->back_pixmap);
+	panel->back_pixmap = NULL;
+
+	/*remove from panels list*/
+	panels = g_slist_remove(panels,w);
+	panel_remove_from_gconf (panel);
+
+	g_free (panel->unique_id);
+	panel->unique_id = NULL;
+
+	if (G_OBJECT_CLASS (panel_widget_parent_class)->finalize)
+		G_OBJECT_CLASS (panel_widget_parent_class)->finalize (obj);
+}
+
 static void
 panel_widget_destroy (GtkObject *obj)
 {
@@ -1826,7 +1858,7 @@ panel_widget_applet_drag_start_no_grab (PanelWidget *panel,
 	g_return_if_fail (ad != NULL);
 
 	if (moving_timeout != 0) {
-		gtk_timeout_remove (moving_timeout);
+		g_source_remove (moving_timeout);
 		moving_timeout = 0;
 		been_moved = FALSE;
 	}
@@ -1863,7 +1895,7 @@ panel_widget_applet_drag_end_no_grab (PanelWidget *panel)
 
 	remove_all_move_bindings (panel);
 	if (moving_timeout != 0) {
-		gtk_timeout_remove (moving_timeout);
+		g_source_remove (moving_timeout);
 		moving_timeout = 0;
 		been_moved = FALSE;
 	}
@@ -1884,16 +1916,23 @@ panel_widget_applet_drag_start (PanelWidget *panel,
 	panel_widget_applet_drag_start_no_grab (panel, applet, drag_off);
 
 	gtk_grab_add (applet);
-	if (applet->window != NULL) {
-		GdkCursor *fleur_cursor = gdk_cursor_new (GDK_FLEUR);
-		gdk_pointer_grab (applet->window,
-				  FALSE,
-				  APPLET_EVENT_MASK,
-				  NULL,
-				  fleur_cursor,
-				  GDK_CURRENT_TIME);
+	if (applet->window) {
+		GdkGrabStatus  status;
+		GdkCursor     *fleur_cursor;
+
+		fleur_cursor = gdk_cursor_new (GDK_FLEUR);
+
+		status = gdk_pointer_grab (applet->window, FALSE,
+					   APPLET_EVENT_MASK, NULL,
+					   fleur_cursor, GDK_CURRENT_TIME);
+
 		gdk_cursor_unref (fleur_cursor);
 		gdk_flush ();
+
+		if (status != GDK_GRAB_SUCCESS) {
+			g_warning (G_STRLOC ": failed to grab pointer");
+			panel_widget_applet_drag_end (panel);
+		}
 	}
 }
 
@@ -1927,49 +1966,6 @@ panel_widget_get_cursorloc (PanelWidget *panel)
 	}
 }
 
-#ifdef FIXME /* This isn't used anymore but looks useful */
-
-/*get amount of free space around the applet (including the applet size),
-  or return 0 on error or if the panel is packed*/
-int
-panel_widget_get_free_space(PanelWidget *panel, GtkWidget *applet)
-{
-	int right,left;
-	GList *li;
-	AppletData *ad;
-
-	g_return_val_if_fail(PANEL_IS_WIDGET(panel), 0);
-	g_return_val_if_fail(GTK_IS_WIDGET(applet), 0);
-	
-	/*this function doesn't make sense on packed panels*/
-	if(panel->packed)
-		return 0;
-	
-	right = panel->size;
-	left = 0;
-	
-	for(li = panel->applet_list; li; li = g_list_next(li)) {
-		ad = li->data;
-		if(ad->applet == applet)
-			break;
-	}
-	/*the applet is not on this panel*/
-	if(!li)
-		return 0;
-	
-	if(li->prev) {
-		AppletData *pad = li->prev->data;
-		left = pad->pos+pad->cells;
-	}
-	if(li->next) {
-		AppletData *nad = li->next->data;
-		right = nad->pos;
-	}
-	
-	return right - left;
-}
-#endif
-
 /*calculates the value to move the applet by*/
 static int
 panel_widget_get_moveby (PanelWidget *panel, int pos, int offset)
@@ -2284,7 +2280,7 @@ schedule_try_move(PanelWidget *panel, gboolean repeater)
 		been_moved = FALSE;
 		panel_widget_applet_move_to_cursor(panel);
 		moving_timeout =
-			gtk_timeout_add (50, move_timeout_handler, panel);
+			g_timeout_add (50, move_timeout_handler, panel);
 	} else
 		been_moved = TRUE;
 }
@@ -2362,15 +2358,16 @@ panel_sub_event_handler(GtkWidget *widget, GdkEvent *event, gpointer data)
 	g_return_val_if_fail(event!=NULL,FALSE);
 
 	switch (event->type) {
-		GdkEventButton *bevent;
 		/*pass these to the parent!*/
 		case GDK_BUTTON_PRESS:
 		case GDK_BUTTON_RELEASE:
-		case GDK_MOTION_NOTIFY:
-			bevent = (GdkEventButton *)event;
-			if(bevent->button != 1 || panel_applet_in_drag)
-				return gtk_widget_event(data, event);
+		case GDK_MOTION_NOTIFY: {
+			GdkEventButton *bevent = (GdkEventButton *)event;
 
+			if (bevent->button != 1 || panel_applet_in_drag)
+				return gtk_widget_event (data, event);
+
+			}
 			break;
 		case GDK_KEY_PRESS:
 			if (panel_applet_in_drag)
@@ -2435,7 +2432,7 @@ panel_widget_applet_destroy (GtkWidget *applet, gpointer data)
 
 
 static void
-bind_top_applet_events(GtkWidget *widget, gboolean bind_lower)
+bind_top_applet_events (GtkWidget *widget)
 {
 	g_return_if_fail(GTK_IS_WIDGET(widget));
 
@@ -2456,7 +2453,7 @@ bind_top_applet_events(GtkWidget *widget, gboolean bind_lower)
 	 * GtkEventBox) for processing by us.
 	 */
 
-	if (bind_lower && GTK_IS_CONTAINER(widget))
+	if (GTK_IS_CONTAINER(widget))
 		gtk_container_foreach (GTK_CONTAINER (widget),
 				       bind_applet_events, widget);
 }
@@ -2519,9 +2516,12 @@ panel_widget_add_forbidden (PanelWidget *panel)
 }
 
 int
-panel_widget_add_full (PanelWidget *panel, GtkWidget *applet, int pos,
-		       gboolean bind_lower_events, gboolean insert_at_pos,
-		       gboolean expand_major, gboolean expand_minor)
+panel_widget_add (PanelWidget *panel,
+		  GtkWidget   *applet,
+		  int          pos,
+		  gboolean     insert_at_pos,
+		  gboolean     expand_major,
+		  gboolean     expand_minor)
 {
 	AppletData *ad = NULL;
 
@@ -2573,7 +2573,7 @@ panel_widget_add_full (PanelWidget *panel, GtkWidget *applet, int pos,
 				   PANEL_APPLET_DATA, ad);
 		
 		/*this is a completely new applet, which was not yet bound*/
-		bind_top_applet_events (applet, bind_lower_events);
+		bind_top_applet_events (applet);
 	}
 
 	panel->applet_list =
@@ -2633,46 +2633,6 @@ panel_widget_reparent (PanelWidget *old_panel,
 	return pos;
 }
 
-#ifdef FIXME /* This isn't used anymore but looks useful */
-int
-panel_widget_move (PanelWidget *panel, GtkWidget *applet, int pos)
-{
-	AppletData *ad;
-
-	g_return_val_if_fail(PANEL_IS_WIDGET(panel),-1);
-	g_return_val_if_fail(GTK_IS_WIDGET(applet),-1);
-	g_return_val_if_fail(pos>=0,-1);
-
-	ad = g_object_get_data (G_OBJECT (applet), PANEL_APPLET_DATA);
-	ad->pos = -1;
-	panel->applet_list = g_list_remove (panel->applet_list, ad);
-	
-	if(pos < 0)
-		pos = 0;
-
-	if(get_applet_list_pos(panel,pos)) 
-		/*this is a slight hack so that this applet is
-		  inserted AFTER an applet with this pos number*/
-		pos++;
-	if(pos==-1) return -1;
-
-	ad->pos = pos;
-	/*reset size to 1*/
-	ad->cells = 1;
-	ad->min_cells = 1;
-	panel->applet_list =
-		g_list_insert_sorted(panel->applet_list,ad,
-				     (GCompareFunc)applet_data_compare);
-
-	g_signal_emit (G_OBJECT(panel),
-		       panel_widget_signals[APPLET_MOVE_SIGNAL],
-		       0, ad->applet);
-	gtk_widget_queue_resize(GTK_WIDGET(panel));
-
-	return pos;
-}
-#endif
-
 void
 panel_widget_change_params(PanelWidget *panel,
 			   GtkOrientation orient,
@@ -2768,60 +2728,53 @@ static void
 panel_widget_push_move_applet (PanelWidget     *panel,
                                GtkDirectionType dir)
 {
-	AppletData       *ad;
-	gint increment = MOVE_INCREMENT;
+	AppletData *applet;
+	int         increment = 0;
 
-	ad = panel->currently_dragged_applet;
-
-	g_return_if_fail (ad);
+	applet = panel->currently_dragged_applet;
+	g_return_if_fail (applet);
 
 	switch (dir) {
 	case GTK_DIR_LEFT:
 	case GTK_DIR_UP:
-		increment = -increment;
+		increment = -MOVE_INCREMENT;
 		break;
 	case GTK_DIR_RIGHT:
 	case GTK_DIR_DOWN:
+		increment = MOVE_INCREMENT;
 		break;
 	default:
 		return;
 	}
-	panel_widget_push_move (panel, ad, increment);
+
+	panel_widget_push_move (panel, applet, increment);
 }
 
 static void 
 panel_widget_switch_move_applet (PanelWidget      *panel,
                                  GtkDirectionType  dir)
 {
-	AppletData       *ad;
-	AppletData *nad = NULL;
-	GList *list;
-	gint increment;
+	AppletData *applet;
+	GList      *list;
 
-	ad = panel->currently_dragged_applet;
+	applet = panel->currently_dragged_applet;
+	g_return_if_fail (applet != NULL);
 
-	g_return_if_fail (ad);
+	list = g_list_find (panel->applet_list, applet);
+	g_return_if_fail (list != NULL);
 
-	list = g_list_find (panel->applet_list, ad);
-	g_return_if_fail (list);
 	switch (dir) {
 	case GTK_DIR_LEFT:
 	case GTK_DIR_UP:
-		if (list->prev)
-			nad = list->prev->data;
+		panel_widget_switch_applet_left (panel, list);
 		break;
 	case GTK_DIR_RIGHT:
 	case GTK_DIR_DOWN:
-		if (list->next)
-			nad = list->next->data;
+		panel_widget_switch_applet_right (panel, list);
 		break;
 	default:
 		return;
 	}
-	if (nad) {
-		increment = nad->pos - ad->pos;
-		panel_widget_switch_move (panel, ad, increment);
-	}
 }
 
 static void 
@@ -2847,6 +2800,7 @@ panel_widget_free_move_applet (PanelWidget      *panel,
 	default:
 		return;
 	}
+
 	pos = panel_widget_get_free_spot (panel, ad, 
                                           increment + ad->pos + ad->drag_off);
 	panel_widget_nice_move (panel, ad, pos);
@@ -2862,16 +2816,8 @@ panel_widget_tab_move (PanelWidget *panel,
 
 	ad = panel->currently_dragged_applet;
 
-	if (!ad) {
-		g_return_if_fail (GTK_WIDGET_HAS_FOCUS (panel));
-		
-		GTK_WIDGET_UNSET_FLAGS (GTK_WIDGET (panel), GTK_CAN_FOCUS);
-		gtk_widget_child_focus (GTK_WIDGET (panel),
-					next ?
-						GTK_DIR_TAB_FORWARD :
-						GTK_DIR_TAB_BACKWARD);
+	if (!ad)
 		return;	
-	}
 	
 	for (li = panels; li; li = li->next) {
 		PanelWidget *previous_panel = NULL;
@@ -2904,38 +2850,37 @@ panel_widget_end_move (PanelWidget *panel)
 static gboolean
 panel_widget_focus_in_event (GtkWidget *widget, GdkEventFocus *event)
 {
-	GtkWidgetClass *class;
-	GtkBindingSet *binding_set;
+	/*
+	 * FIXME: we need to figure out something sensible
+	 *        to do to indicate focus
+	 */
 
-	gtk_widget_set_state (widget, GTK_STATE_PRELIGHT);
-
-	class = GTK_WIDGET_GET_CLASS (widget);
-	binding_set = gtk_binding_set_by_class (class);
-	add_tab_bindings (binding_set, 0, TRUE);
-	add_tab_bindings (binding_set, GDK_SHIFT_MASK, FALSE);
 	GTK_WIDGET_CLASS (panel_widget_parent_class)->focus_in_event (widget, event);
+
 	return FALSE;
 }
 
 static gboolean
 panel_widget_focus_out_event (GtkWidget *widget, GdkEventFocus *event)
 {
-	gtk_widget_set_state (widget, GTK_STATE_NORMAL);
-
-	if (GTK_WINDOW (PANEL_WIDGET (widget)->panel_parent)->has_focus) {
-		GtkWidgetClass *class;
-		GtkBindingSet *binding_set;
-
+	if (GTK_WINDOW (PANEL_WIDGET (widget)->panel_parent)->has_focus)
 		GTK_WIDGET_UNSET_FLAGS (widget, GTK_CAN_FOCUS);
-		class = GTK_WIDGET_GET_CLASS (widget);
-		binding_set = gtk_binding_set_by_class (class);
-		remove_tab_bindings (binding_set, 0, TRUE);
-		remove_tab_bindings (binding_set, GDK_SHIFT_MASK, FALSE);
-	}
+
 	GTK_WIDGET_CLASS (panel_widget_parent_class)->focus_out_event (widget, event);
+
 	return FALSE;
 }
 
+static gboolean
+panel_widget_real_focus (GtkWidget        *widget,
+                         GtkDirectionType  direction)
+{
+	if (GTK_WIDGET_HAS_FOCUS (widget))
+		GTK_WIDGET_UNSET_FLAGS (widget, GTK_CAN_FOCUS);
+
+	return GTK_WIDGET_CLASS (panel_widget_parent_class)->focus (widget, direction);
+}
+
 void panel_widget_focus (PanelWidget *panel)
 {
 	/*
